''' (c) 2008 by the GunGame Coding Team

    Title: gg_knife_nade_bonus
    Version: 1.0.0
    Description: Custom addon.
    '''

# ==============================================================================
#   IMPORTS
# ==============================================================================
# Python imports

# EventScripts imports
import es
import gamethread
import playerlib

# GunGame imports
import gungamelib

# ==============================================================================
#   ADDON REGISTRATION
# ==============================================================================
# Register this addon with EventScripts
info = es.AddonInfo()
info.name     = 'Knife and Nade level bonuses for GunGame 5'
info.version  = '1.0.0'
info.url      = 'http://forums.gungame5.com/viewtopic.php?f=17&t=326'
info.basename = 'gungame/custom_addons/gg_knife_nade_bonus'
info.author   = 'Sc0pE'

es.ServerVar('gg_knife_nade_bonus_ver', info.version).makepublic()

# ==============================================================================
#   GLOBALS
# ==============================================================================
hegrenade_list = []
knife_list = []

hegrenade_weapons = gungamelib.getVariableValue('gg_KNB_hegrenade_weapons')
knife_weapons = gungamelib.getVariableValue('gg_KNB_knife_weapons')
if hegrenade_weapons:
    list_hegrenade_weapons = hegrenade_weapons.split(',')
else:
    list_hegrenade_weapons = 0

if knife_weapons:
    list_knife_weapons = knife_weapons.split(',')
else:
    list_knife_weapons = 0

items_spawn_only = gungamelib.getVariableValue('gg_KNB_items_spawn_only')
health_spawn_only = gungamelib.getVariableValue('gg_KNB_health_spawn_only')

hegrenade_health = gungamelib.getVariableValue('gg_KNB_hegrenade_health')
hegrenade_speed = gungamelib.getVariableValue('gg_KNB_hegrenade_speed')
hegrenade_color = (gungamelib.getVariableValue('gg_KNB_hegrenade_red'), gungamelib.getVariableValue('gg_KNB_hegrenade_green'), gungamelib.getVariableValue('gg_KNB_hegrenade_blue'), gungamelib.getVariableValue('gg_KNB_hegrenade_alpha'))
hegrenade_sparks = gungamelib.getVariableValue('gg_KNB_hegrenade_sparks')

knife_health = gungamelib.getVariableValue('gg_KNB_knife_health')
knife_speed = gungamelib.getVariableValue('gg_KNB_knife_speed')
knife_color = (gungamelib.getVariableValue('gg_KNB_knife_red'), gungamelib.getVariableValue('gg_KNB_knife_green'), gungamelib.getVariableValue('gg_KNB_knife_blue'), gungamelib.getVariableValue('gg_KNB_knife_alpha'))
knife_sparks = gungamelib.getVariableValue('gg_KNB_knife_sparks')

# ==============================================================================
#   GAME EVENTS
# ==============================================================================
def load():
    # Register this addon with GunGame
    gg_knife_nade_bonus = gungamelib.registerAddon('gg_knife_nade_bonus')
    gg_knife_nade_bonus.setDisplayName('GG Knife/Nade Bonus')
    gg_knife_nade_bonus.addDependency('gg_nade_bonus', 0)

def unload():
    # Unregister this addon with GunGame
    gungamelib.unregisterAddon('gg_knife_nade_bonus')

def player_spawn(event_var):
    if not gungamelib.getGlobal('isWarmup'):
        userid = int(event_var['userid'])
        gamethread.delayed(0, weaponCheck, (userid, gungamelib.getPlayer(userid).getWeapon(), 1))

def gg_levelup(event_var):
    attacker = int(event_var['attacker'])
    removePlayer(attacker, 1)
    weaponCheck(attacker, gungamelib.getPlayer(attacker).getWeapon(), 0)

def gg_leveldown(event_var):
    userid = int(event_var['userid'])
    removePlayer(userid, 1)
    weaponCheck(userid, gungamelib.getPlayer(userid).getWeapon(), 0)

def player_disconnect(event_var):
    removePlayer(int(event_var['userid']), 0)

def player_death(event_var):
    removePlayer(int(event_var['userid']), 1)

# ==============================================================================
#   MAIN HELPER FUNCTIONS
# ==============================================================================
def weaponCheck(userid, new_weapon, just_spawned):
    if new_weapon == 'hegrenade':
        startHegrenadeEffects(userid, just_spawned)
    elif new_weapon == 'knife':
        startKnifeEffects(userid, just_spawned)

def removePlayer(userid, end_effects):
    userid = userid
    if userid in hegrenade_list:
        if end_effects:
            endHegrenadeEffects(userid)
        hegrenade_list.remove(userid)
    if userid in knife_list:
        if end_effects:
            endKnifeEffects(userid)
        knife_list.remove(userid)

def startHegrenadeEffects(userid, just_spawned):
    if not userid in hegrenade_list:
        hegrenade_list.append(userid)
    if len(list_hegrenade_weapons):
        giveWeaponsList(userid, list_hegrenade_weapons, just_spawned)
    if hegrenade_health:
        setHealth(userid, hegrenade_health, just_spawned)
    if hegrenade_speed:
        setSpeed(userid, hegrenade_speed)
    if hegrenade_sparks:
        startSparks(userid)
    setColor(userid, hegrenade_color)

def startKnifeEffects(userid, just_spawned):
    if not userid in knife_list:
        knife_list.append(userid)
    if list_knife_weapons:
        giveWeaponsList(userid, list_knife_weapons, just_spawned)
    if knife_health:
        setHealth(userid, knife_health, just_spawned)
    if knife_speed:
        setSpeed(userid, knife_speed)
    if knife_sparks:
        startSparks(userid)
    setColor(userid, knife_color)

def endHegrenadeEffects(userid):
    if hegrenade_speed:
        setSpeed(userid, 1)
    if hegrenade_sparks:
        endSparks(userid)
    setColor(userid, (255, 255, 255, 255))

def endKnifeEffects(userid):
    if knife_speed:
        setSpeed(userid, 1)
    if knife_sparks:
        endSparks(userid)
    setColor(userid, (255, 255, 255, 255))

# ==============================================================================
#   OTHER HELPER FUNCTIONS
# ==============================================================================
def giveWeaponsList(userid, weapons_list, just_spawned):
    if items_spawn_only and not just_spawned:
        return
    gungamePlayer = gungamelib.getPlayer(userid)
    for weapon in weapons_list:
        gungamePlayer.give('weapon_%s' % weapon)
    es.delayed(0, 'es_xsexec %s "use weapon_%s"' % (userid, gungamePlayer.getWeapon()))

def setHealth(userid, health, just_spawned):
    if health_spawn_only and not just_spawned:
        return
    es.setplayerprop(userid, "CBasePlayer.m_iHealth", health)

def setSpeed(userid, speed):
    es.setplayerprop(userid, "CBasePlayer.localdata.m_flLaggedMovementValue", speed)

def setColor(userid, color_tuple):
    playerlib.getPlayer(userid).set('color', color_tuple)

def startSparks(userid):
    es.server.cmd('es_xgive %s env_spark' % userid)
    es.server.cmd('es_xfire %s env_spark SetParent !activator' % userid)
    es.server.cmd('es_xfire %s env_spark AddOutput "spawnflags 896"' % userid)
    es.server.cmd('es_xfire %s env_spark AddOutput "angles -90 0 0"' % userid)
    es.server.cmd('es_xfire %s env_spark AddOutput "magnitude 8"' % userid)
    es.server.cmd('es_xfire %s env_spark AddOutput "traillength 3"' % userid)
    es.server.cmd('es_xfire %s env_spark StartSpark' % userid)

def endSparks(userid):
    es.server.cmd('es_xfire %s env_spark StopSpark' % userid)
    es.server.cmd('es_xfire %s env_spark Kill' % userid)